@@ -43,24 +43,34 @@
 #define PATH_MAILLOG	"/var/adm/syslog.dated/current/mail.log"
 #endif
 
-#ifdef SUNOS
+#ifdef SOLARIS
+#define HAVE_SNPRINTF
+#define HAVE_VSNPRINTF
 #define HAVE_GETRUSAGE
 #define PID_T_IS_LONG_INT
-#if 0
-/*
- * Do not use socklen_t, as I found it in the Solaris 7 manual page but
- * not for Solaris 2.4. Play it safe and use the "int" default below.
- */
-#define SOCKLEN_T	socklen_t
+#define SOCKLEN_T       socklen_t
+#define PATH_TMP        "/tmp"
+#define PATH_VAR_TMP    "/var/tmp"
+#define PATH_PASSWD     "/etc/passwd"
+#define PATH_WTMP       "/var/adm/wtmpx"
+#define PATH_UTMP       "/var/adm/utmpx"
+#define PATH_SYSLOG     "/var/log/syslog"
 #endif
+
+
+#ifdef SOLARIS_OLD
+#define HAVE_GETRUSAGE
+#define PID_T_IS_LONG_INT
+#define SOCKLEN_T       int
 #define PATH_TMP        "/tmp"
 #define PATH_VAR_TMP    "/var/tmp"
 #define PATH_PASSWD     "/etc/passwd"
 #define PATH_WTMP       "/var/adm/wtmp"
-#define PATH_UTMP       "/etc/utmp"
+#define PATH_UTMP       "/var/adm/utmp"
 #define PATH_SYSLOG     "/var/log/syslog"
 #endif
 
+
 #ifdef NEXTSTEP3
 #define HAVE_GETRUSAGE
 #define SIZEOF_IS_LONG_INT
@@ -86,7 +96,7 @@
 #define PATH_SYSLOG     "/private/adm/messages"
 #endif
 
-#ifdef IRIX6
+#ifdef IRIX62
 #define HAVE_GETRUSAGE
 #define PID_T_IS_LONG_INT
 #define PATH_TMP        "/tmp"
@@ -97,6 +107,19 @@
 #define PATH_SYSLOG     "/var/adm/SYSLOG"
 #endif
 
+#ifdef IRIX65
+#define HAVE_GETRUSAGE
+#define HAVE_SNPRINTF
+#define HAVE_VSNPRINTF
+#define PID_T_IS_LONG_INT
+#define PATH_TMP        "/tmp"
+#define PATH_VAR_TMP	"/var/tmp"
+#define PATH_PASSWD     "/etc/passwd"
+#define PATH_WTMP       "/var/adm/wtmpx"
+#define PATH_UTMP       "/var/adm/utmpx"
+#define PATH_SYSLOG     "/var/adm/SYSLOG"
+#endif
+
 #ifdef AIX43
 #define HAVE_GETRUSAGE
 #if defined(__64BIT_KERNEL) && defined(_KERNEL)
