@@ -10,12 +10,10 @@
 #include <syslog.h>
 #include <pwd.h>
 #include <sys/stat.h>
-#include <sys/socket.h>
-#include <sys/un.h>
-#include <sys/time.h>
 #include <sys/types.h>
+#include <sys/socket.h>
+#include <sys/time.h>
 #include <sys/wait.h>
-#include <netinet/in.h>
 
 #include "prngd.h"
 
@@ -198,7 +196,7 @@ static void usage(const char *progname)
 {
   (void)fprintf(stderr, "prngd %s (%s)\n", PRNGD_VERSION, PRNGD_DATE);
   (void)fprintf(stderr,
-	  "Usage: %s [options] /path/to/socket1 [/path/to/socket2 ...]\nOptions:\n\t-d/--debug: debugging on\n\t-c/--cmdfile cmdpath: use cmdpath for entropy commands [%s]\n\t-s/--seedfile seedpath: use seedpath as seedfile [%s]\n\t-n/--no-seedfile: no seedfile, keep pool in memory only\n\t-m/--mode mode: use mode for sockets [%04o]\n\t-k/--kill: kill daemon on other side\n\t-v/--version: print version and exit\n",
+	  "Usage: %s [options] (/path/to/socket1 | tcp/localhost:port) [(/path/to/socket2 | tcp/localhost:port)] ... \nOptions:\n\t-d/--debug: debugging on\n\t-c/--cmdfile cmdpath: use cmdpath for entropy commands [%s]\n\t-s/--seedfile seedpath: use seedpath as seedfile [%s]\n\t-n/--no-seedfile: no seedfile, keep pool in memory only\n\t-m/--mode mode: use mode for sockets [%04o]\n\t-k/--kill: kill daemon on other side\n\t-v/--version: print version and exit\n",
 	  progname, CONFIGFILE, RANDSAVENAME, SOCKETMODE);
 }
 
@@ -207,18 +205,19 @@ int main(int argc, char *argv[])
   int argp;
   int ret;
   int statval;
-  int killmode = 0;
+  int killmode = 0, killfail = 0;
   time_t seed_stat_interval = SEED_STAT_INTERVAL;
   time_t seed_ext_interval = SEED_EXT_INTERVAL;
-  int i, numsock = 0;
-  char **randsock = NULL;
+  int i, numsock_file = 0, numsock_tcp = 0;
+  char **randsock_tcp = NULL;
+  char **randsock_file = NULL;
   char *cmdpath = CONFIGFILE;
   char *seedpath = RANDSAVENAME;
   char msg_buf[1024];
   unsigned int socketmode = SOCKETMODE;
-  int *service_socket, kill_socket;
-  struct sockaddr_un sockun, sockun_kill;
-  struct stat randsock_stat;
+  int *service_socket, service_count = 0;
+  int kill_socket;
+  struct stat randsock_file_stat;
   struct passwd *pwent;
   char *username = "unknown";
   entropy_source_t *entropy_source;
@@ -257,17 +256,30 @@ int main(int argc, char *argv[])
       seedpath = NULL;
     else if (!strcmp(argv[argp], "-k") || !strcmp(argv[argp], "--kill"))
       killmode = 1;
-    else if (argv[argp][0] != '-') {
-      if (numsock == 0)
-        randsock = malloc(sizeof(char *));
+    else if (argv[argp][0] == '/') {
+      if (numsock_file == 0)
+        randsock_file = malloc(sizeof(char *));
       else
-	randsock = realloc(randsock, (numsock + 1) * sizeof(char *));
-      if (randsock == NULL) {
+	randsock_file = realloc(randsock_file,
+				(numsock_file + 1) * sizeof(char *));
+      if (randsock_file == NULL) {
 	error_fatal("Could not allocate memory", errno);
 	exit(EXIT_FAILURE);
       }
-      randsock[numsock++] = argv[argp];
+      randsock_file[numsock_file++] = argv[argp];
     }
+    else if (!strncmp(argv[argp], "tcp/localhost:", strlen("tcp/localhost:"))) {
+      if (numsock_tcp == 0)
+        randsock_tcp = malloc(sizeof(char *));
+      else
+	randsock_tcp = realloc(randsock_tcp,
+				(numsock_tcp + 1) * sizeof(char *));
+      if (randsock_tcp == NULL) {
+	error_fatal("Could not allocate memory", errno);
+	exit(EXIT_FAILURE);
+      }
+      randsock_tcp[numsock_tcp++] = argv[argp];
+    } 
     else
     {
       usage(argv[0]);
@@ -275,19 +287,15 @@ int main(int argc, char *argv[])
     }
     argp++;
   }
-  if (!numsock) {
+  if ((numsock_file == 0) && (numsock_tcp == 0)) {
     usage(argv[0]);
+    error_fatal("No listen socket given, exiting!", 0);
     exit(EXIT_FAILURE);
   }
 
   if (!killmode)
     parse_configfile(cmdpath, &entropy_source);
 
-  if (!numsock)
-  {
-    error_fatal("no socket name given, exiting!", 0);
-    exit(EXIT_FAILURE);
-  }
   if (debug)
     (void)fprintf(stdout, "Debugging enabled\n");
 
@@ -324,25 +332,89 @@ int main(int argc, char *argv[])
     count_openfd++;
   }
 
-  if (!(service_socket = malloc(numsock * sizeof(int)))) {
+  if (!(service_socket = malloc((numsock_file + numsock_tcp) * sizeof(int)))) {
     error_fatal("Could not allocate memory", errno);
     exit(EXIT_FAILURE);
   }
 
-  /*
-   * Prepare the socket used for killing or testing for a running
-   * daemon.
-   */
-  kill_socket = socket(AF_UNIX, SOCK_STREAM, 0);
-  if (kill_socket < 0) {
-    error_fatal( "Could not create socket", errno);
-    exit(EXIT_FAILURE);
+  if (killmode) {
+    for (i = 0; i < numsock_file; i++) {
+
+    /*
+     * Try to kill daemons on the other side of a unix socket. If we cannot
+     * connect to a daemon, we set a flag but continue, since there may be
+     * other daemons waiting for kill.
+     */
+      statval = stat(randsock_file[i], &randsock_file_stat);
+      if ((statval < 0) && (errno != ENOENT)) {
+        (void)snprintf(msg_buf, sizeof(msg_buf),
+		       "Cannot stat socket position %s", randsock_file[i]);
+        msg_buf[sizeof(msg_buf) - 1] = '\0';
+        error_error(msg_buf, errno);
+        killfail = 1;
+        continue;
+      }
+      if ((statval < 0) && (errno == ENOENT)) {
+        (void)snprintf(msg_buf, sizeof(msg_buf), "No object at position %s",
+        	randsock_file[i]);
+        msg_buf[sizeof(msg_buf) - 1] = '\0';
+        error_fatal(msg_buf, errno);
+        killfail = 1;
+        continue;
+      }
+      if (!S_ISSOCK(randsock_file_stat.st_mode)) {
+	(void)snprintf(msg_buf, sizeof(msg_buf), "Will not touch %s: no socket",
+		randsock_file[i]);
+	msg_buf[sizeof(msg_buf) - 1] = '\0';
+	error_error(msg_buf, errno);
+	killfail = 1;
+	continue;
+      }
+
+      if ((kill_socket = unix_connect(randsock_file[i], 0, 1)) < 0) {
+	    (void)snprintf(msg_buf, sizeof(msg_buf),
+	    	"cannot kill peer at %s: no daemon", randsock_file[i]);
+	    msg_buf[sizeof(msg_buf) - 1] = '\0';
+	    error_error(msg_buf, 0);
+	    killfail = 1;
+      }
+      else {
+	killpeer(kill_socket);
+	my_shutdown(kill_socket, SHUT_RDWR);
+	(void)close(kill_socket);
+      }
+    }
+
+    for (i = 0; i < numsock_tcp; i++) {
+      if ((kill_socket = tcp_connect(randsock_tcp[i], 0, 1)) < 0) {
+	(void)snprintf(msg_buf, sizeof(msg_buf),
+		       "cannot kill peer at %s: no daemon", randsock_tcp[i]);
+	msg_buf[sizeof(msg_buf) - 1] = '\0';
+	error_error(msg_buf, 0);
+	/*
+	 * Do no exit because there may be more work to do!
+	 */
+      }
+      else {
+	killpeer(kill_socket);
+	my_shutdown(kill_socket, SHUT_RDWR);
+	(void)close(kill_socket);
+      }
+    }
+    /*
+     * Everything has been done!
+     */
+    if (killfail)
+      exit(EXIT_FAILURE);
+    else
+      exit(EXIT_SUCCESS);
   }
-  count_openfd++;
+
+
   /*
-   * Set up the sockets to service entropy requests
+   * Set up the UNIX domain sockets to service entropy requests
    */
-  for (i = 0; i < numsock; i++) {
+  for (i = 0; i < numsock_file; i++) {
 
     /*
      * Check out what is already available at the place of the future
@@ -352,83 +424,38 @@ int main(int argc, char *argv[])
      * socket! If removal fails for whatever reason, stop, as something
      * may be seriously wrong.
      */
-    statval = stat(randsock[i], &randsock_stat);
+    statval = stat(randsock_file[i], &randsock_file_stat);
     if ((statval < 0) && (errno != ENOENT)) {
       (void)snprintf(msg_buf, sizeof(msg_buf), "Cannot stat socket position %s",
-      	randsock[i]);
+      	randsock_file[i]);
       msg_buf[sizeof(msg_buf) - 1] = '\0';
       error_fatal(msg_buf, errno);
       exit(EXIT_FAILURE);
     }
     else if (!statval) {
-      if (!S_ISSOCK(randsock_stat.st_mode)) {
+      if (!S_ISSOCK(randsock_file_stat.st_mode)) {
 	(void)snprintf(msg_buf, sizeof(msg_buf), "Will not touch %s: no socket",
-		randsock[i]);
+		randsock_file[i]);
 	msg_buf[sizeof(msg_buf) - 1] = '\0';
 	error_fatal(msg_buf, errno);
 	exit(EXIT_FAILURE);
       } else {
-	/*
-	 * Now that we have found a socket, we must check whether there
-	 * is a daemon listining on it. When in kill mode, this is great
-	 * and we are going to kill it. When not in kill mode, we cannot
-	 * start a new daemon at this address and exit with error.
-	 */
-	(void)memset(&sockun, 0, sizeof(sockun_kill));
-	sockun_kill.sun_family = AF_UNIX;
-	if (sizeof(sockun_kill.sun_path) < strlen(randsock[i]) + 1) {
-#ifdef SIZEOF_IS_LONG_INT
-	(void)snprintf(msg_buf, sizeof(msg_buf),
-		"Sockfilename too long: %s, maxlen=%ld\n", randsock[i],
-		sizeof(sockun_kill.sun_path));
-#else
-	(void)snprintf(msg_buf, sizeof(msg_buf),
-		"Sockfilename too long: %s, maxlen=%d\n", randsock[i],
-		sizeof(sockun_kill.sun_path));
-#endif /* SIZEOF_IS_LONG_INT */
-	msg_buf[sizeof(msg_buf) - 1] = '\0';
-	error_error(msg_buf, 0);
-	  exit(EXIT_FAILURE);
-        }
-
-	(void)strcpy(sockun_kill.sun_path, randsock[i]);
-	if (connect(kill_socket, (struct sockaddr *) &sockun_kill,
-		    sizeof(sockun_kill)) < 0) {
-	  if (killmode) {
-	    (void)snprintf(msg_buf, sizeof(msg_buf),
-	    	"cannot kill peer at %s: no daemon", randsock[i]);
-	    msg_buf[sizeof(msg_buf) - 1] = '\0';
-	    error_error(msg_buf, 0);
-	    /*
-	     * Do no exit because there may be more work to do!
-	     */
-	    continue;
-	  }
-	}
-	else if (killmode) {
-	  killpeer(kill_socket);
-	  my_shutdown(kill_socket, SHUT_RDWR);
-	  count_openfd--;
-	  continue;
-	}
-	else {
+	if ((kill_socket = unix_connect(randsock_file[i], 0, 1)) >= 0) {
 	  (void)snprintf(msg_buf, sizeof(msg_buf),
 		  "socket %s already used by another process, exiting.\n",
-		  randsock[i]);
+		  randsock_file[i]);
 	  msg_buf[sizeof(msg_buf) - 1] = '\0';
 	  error_fatal(msg_buf, 0);
 	  exit(EXIT_FAILURE);
 	}
-	my_shutdown(kill_socket, SHUT_RDWR);
-	count_openfd--;
 
 	/*
          * Now there is a socket without service listening on it: we can
 	 * remove it.
 	 */
-	if (remove(randsock[i]) < 0) {
+	if (remove(randsock_file[i]) < 0) {
 	  (void)snprintf(msg_buf, sizeof(msg_buf),
-	  	"Cannot remove already existing entry %s", randsock[i]);
+	  	"Cannot remove already existing entry %s", randsock_file[i]);
 	  msg_buf[sizeof(msg_buf) - 1] = '\0';
 	  error_fatal(msg_buf, errno);
 	  exit(EXIT_FAILURE);
@@ -436,61 +463,16 @@ int main(int argc, char *argv[])
       }
     }
 
-    /*
-     * We can never reach this point in killmode, as for whatever condition
-     * not forcing immediate abort the loop was "continue"ed before.
-     */
-    assert(!killmode);
-
-    service_socket[i] = socket(AF_UNIX, SOCK_STREAM, 0);
-    if (service_socket[i] < 0)
-    {
-      error_fatal("Could not create socket", errno);
-      exit(EXIT_FAILURE);
-    }
-    /*
-     * Bind the socket to the actual socket-file
-     */
-    (void)memset(&sockun, 0, sizeof(sockun));
-    sockun.sun_family = AF_UNIX;
-    (void)strcpy(sockun.sun_path, randsock[i]);
-    if (bind(service_socket[i], (struct sockaddr *) &sockun,
-	     sizeof(sockun)) < 0)
-    {
-      (void)snprintf(msg_buf, sizeof(msg_buf), "Could not bind socket to %s",
-      	randsock[i]);
-      msg_buf[sizeof(msg_buf) - 1] = '\0';
-      error_fatal(msg_buf, errno);
-      exit(EXIT_FAILURE);
-    }
-    if (chmod(randsock[i], socketmode) < 0)
-    {
-      (void)snprintf(msg_buf, sizeof(msg_buf), "Could not chmod socket to %04o",
-      	socketmode);
-      msg_buf[sizeof(msg_buf) - 1] = '\0';
-      error_fatal(msg_buf, errno);
-      exit(EXIT_FAILURE);
-    }
-    if (listen(service_socket[i], SOMAXCONN) < 0)
-    {
-      error_fatal("Could not listen on socket", errno);
-      exit(EXIT_FAILURE);
-    }
-    /*
-     * Switch the service_socket to non-blocking mode (otherwise it might
-     * hang inside accept() if a connection is closed between select() and
-     * accept().
-     */
-    non_blocking(service_socket[i], 1);
-    close_on_exec(service_socket[i], 1);
+    service_socket[service_count++] = unix_listen(randsock_file[i], 1, 1);
     count_openfd++;
   }
 
-  if (killmode) {
-    /*
-     * Everything has been done!
-     */
-    exit(EXIT_SUCCESS);
+  /*
+   * Set up the TCP sockets to service entropy requests
+   */
+  for (i = 0; i < numsock_tcp; i++) {
+    service_socket[service_count++] = tcp_listen(randsock_tcp[i], 1, 1);
+    count_openfd++;
   }
 
 
@@ -530,7 +512,7 @@ int main(int argc, char *argv[])
   seed_internal(NULL);
 
   /* Never returns */
-  main_loop(service_socket, numsock, sockun, seed_stat_interval,
+  main_loop(service_socket, service_count, seed_stat_interval,
 	    seed_ext_interval, entropy_source, MAX_GATHERER_BYTES);
 
   /*
