@@ -82,13 +82,14 @@ static void child_handler(int received_signal /*ARGSUSED*/)
   errno = save_errno;
 }
 
-void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
-		      time_t seed_stat_interval, time_t seed_ext_interval,
-		      entropy_source_t *entropy_source, int max_gatherer_bytes)
+void main_loop(int *service_socket, int numsock, time_t seed_stat_interval,
+	       time_t seed_ext_interval, entropy_source_t *entropy_source,
+	       int max_gatherer_bytes)
 {
   int i;
   int accept_fd;
-  SOCKLEN_T sockunlen;
+  struct sockaddr addr;
+  SOCKLEN_T addrlen;
   int num, numbytes, error;
   int pipedes[2];
   time_t last_seed_stat = 0, last_seed_ext = 0;
@@ -587,9 +588,8 @@ void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
       }
       if (FD_ISSET(service_socket[i], &rfds))
       {
-        sockunlen = sizeof(sockun);
-        accept_fd = accept(service_socket[i], (struct sockaddr *) &sockun,
-			   &sockunlen);
+        addrlen = sizeof(addr);
+        accept_fd = accept(service_socket[i], &addr, &addrlen);
         if (accept_fd < 0)
 	  continue;		/*
 				 * Ignore and stay alive 
